@@ -84,7 +84,6 @@ class Cursor(object):
 
         self.tzinfo_factory = tz.FixedOffsetTimezone
         self.row_factory = row_factory
-        self.itersize = 2000
 
         self._closed = False
         self._description = None
@@ -404,7 +403,7 @@ class Cursor(object):
 
     @check_closed
     @check_async
-    def copy_from(self, file, table, sep='\t', null='\N', size=8192,
+    def copy_from(self, file, table, sep='\t', null='\\N', size=8192,
                   columns=None):
         """Reads data from a file-like object appending them to a database
         table (COPY table FROM file syntax).
@@ -419,20 +418,22 @@ class Cursor(object):
         else:
             columns_str = ''
 
-        query = "COPY %s%s FROM stdin WITH DELIMITER AS %s" % (
-            table, columns_str, util.quote_string(self._conn, sep))
-        if null:
-            query += " NULL AS %s" % util.quote_string(self._conn, null)
+        query = "COPY %s%s FROM stdin WITH DELIMITER AS %s NULL AS %s" % (
+            table, columns_str,
+            util.quote_string(self._conn, sep),
+            util.quote_string(self._conn, null))
 
         self._copysize = size
         self._copyfile = file
-        self._pq_execute(query)
-        self._copyfile = None
-        self._copysize = None
+        try:
+            self._pq_execute(query)
+        finally:
+            self._copyfile = None
+            self._copysize = None
 
     @check_closed
     @check_async
-    def copy_to(self, file, table, sep='\t', null='\N', columns=None):
+    def copy_to(self, file, table, sep='\t', null='\\N', columns=None):
         """Writes the content of a table to a file-like object (COPY table
         TO file syntax).
 
@@ -446,18 +447,20 @@ class Cursor(object):
         else:
             columns_str = ''
 
-        query = "COPY %s%s TO stdout WITH DELIMITER AS %s" % (
-            table, columns_str, util.quote_string(self._conn, sep))
-        if null:
-            query += " NULL AS %s" % util.quote_string(self._conn, null)
+        query = "COPY %s%s TO stdout WITH DELIMITER AS %s NULL AS %s" % (
+            table, columns_str,
+            util.quote_string(self._conn, sep),
+            util.quote_string(self._conn, null))
 
         self._copyfile = file
-        self._pq_execute(query)
-        self._copyfile = None
+        try:
+            self._pq_execute(query)
+        finally:
+            self._copyfile = None
 
     @check_closed
     @check_async
-    def copy_expert(self, sql, file, size=8196):
+    def copy_expert(self, sql, file, size=8192):
         if not sql:
             return
 
@@ -466,8 +469,10 @@ class Cursor(object):
                 " COPY FROM; a writeable file-like object for COPY TO.")
 
         self._copyfile = file
-        self._pq_execute(sql)
-        self._copyfile = None
+        try:
+            self._pq_execute(sql)
+        finally:
+            self._copyfile = None
 
     @check_closed
     def setinputsizes(self, sizes):
@@ -635,9 +640,14 @@ class Cursor(object):
             libpq.PQclear(self._pgres)
             self._pgres = None
 
-
     def _pq_execute(self, query, async=False):
+        """Execute the query"""
         pgconn = self._conn._pgconn
+
+        # Check the status of the connection
+        if libpq.PQstatus(pgconn) != libpq.CONNECTION_OK:
+            raise self._conn._create_exception()
+
         if not async:
             with self._conn._lock:
                 if not self._conn._have_wait_callback():
@@ -648,6 +658,7 @@ class Cursor(object):
                     raise self._conn._create_exception(pgres=self._pgres)
                 self._conn._process_notifies()
             self._pq_fetch()
+
         else:
             with self._conn._lock:
                 ret = libpq.PQsendQuery(pgconn, query)
